# -*- coding: utf-8 -*-
from setuphelpers import *
import platform
import json
import psutil

# Usable WAPT package functions: install(), uninstall(), session_setup(), audit(), update_package()
# Declaring global variables - Warnings: 1) WAPT context is only available in package functions; 2) Global variables are not persistent between calls

bin_contains = "OpenJDK11U"
bin_type = "jre"
bin_vm = "temurin-11.jre"


def install():
    # Declaring local variables
    # package_version = control.get_software_version()
    bin_name = glob.glob("*%s*.pkg" % bin_contains)[0]
    # uninstalling previous version
    if isdir("/Library/Java/JavaVirtualMachines/%s" % bin_vm):
        print("suppression de openjdk")
        remove_tree("/Library/Java/JavaVirtualMachines/%s" % bin_vm)

    # Installing the software
    print("Installing: %s" % bin_name)
    install_pkg(bin_name)


def uninstall():
    print("suppression de openjdk")
    remove_tree("/Library/Java/JavaVirtualMachines/%s" % bin_vm)


def update_package():
    # Declaring local variables
    arch = control.architecture
    result = False
    proxies = get_proxies()

    if not proxies:
        proxies = get_proxies_from_wapt_console()
    app_name = control.name
    # api : https://api.adoptium.net/q/swagger-ui/#/Assets/searchReleases
    url_api = "https://api.adoptium.net/v3/assets/feature_releases/11/ga?architecture=x64&heap_size=normal&image_type=jre&jvm_impl=hotspot&os=mac&page=0&page_size=10&project=jdk&sort_method=DEFAULT&sort_order=DESC&vendor=eclipse"

    # Getting latest version information from official sources
    print("API used is: %s" % url_api)
    json_load = json.loads(wgets(url_api, proxies=proxies))

    found = False

    for releases in json_load:
        if found:
            break
        for binary in releases["binaries"]:
            if (
                bin_contains.lower() in binary["installer"]["link"].lower()
                and binary["image_type"].lower() == bin_type.lower()
                and binary["architecture"].lower() == "x64"
                and binary["installer"]["link"].endswith(".pkg")
            ):
                version = binary["scm_ref"].split("jdk-")[1].split("_")[0].replace("+", "0")
                url_dl = binary["installer"]["link"]
                latest_bin = url_dl.rsplit("/", 1)[1]
                found = True
                break

    print("Latest %s version is: %s" % (app_name, version))
    print("Download URL  is: %s" % url_dl)
    # Downloading latest  binaries
    if not isfile(latest_bin):
        print("Downloading: %s" % latest_bin)
        wget(url_dl, latest_bin, proxies=proxies)

    # Changing version of the package
    if Version(version) > Version(control.get_software_version()):
        print("Software version updated (from: %s to: %s)" % (control.get_software_version(), Version(version)))
        result = True
    else:
        print("Software version up-to-date (%s)" % Version(version))
    control.version = "%s-%s" % (Version(version), control.version.split("-", 1)[-1])
    # control.set_software_version(version)
    control.save_control_to_wapt()
    # renaming binary
    os.rename(latest_bin, bin_contains + version + ".pkg")
    # Deleting outdated binaries
    remove_outdated_binaries(version)

    # Validating update-package-sources
    return result


def get_proxies():
    r"""Return system proxy with the urllib python library
    >>> get_proxies()
    {'http': 'http://srvproxy.ad.domain.lan:8080',
    'https': 'http://srvproxy.ad.domain.lan:8080'}
    """
    if platform.python_version_tuple()[0] == "3":
        from urllib.request import getproxies
    else:
        from urllib import getproxies
    return getproxies()


def get_proxies_from_wapt_console():
    r"""Return proxy information from the current user WAPT console
    >>> get_proxies_from_wapt_console()
    {'http': 'http://srvproxy.ad.domain.lan:8080',
    'https': 'http://srvproxy.ad.domain.lan:8080'}
    """
    proxies = {}
    if platform.system() == "Windows":
        waptconsole_ini_path = makepath(user_local_appdata(), "waptconsole", "waptconsole.ini")
    else:
        waptconsole_ini_path = makepath(user_home_directory(), ".config", "waptconsole", "waptconsole.ini")
    if isfile(waptconsole_ini_path):
        proxy_wapt = inifile_readstring(waptconsole_ini_path, "global", "http_proxy")
        if proxy_wapt:
            proxies = {"http": proxy_wapt, "https": proxy_wapt}
    return proxies


def get_version_from_binary(filename, property_name="ProductVersion"):
    r"""Get installer version from file informations, for now, only exe and msi files are compatibles
    Args:
        filename (str): path to the file
        property_name (str): selected property
    Returns:
        str: version number
    """
    if filename.endswith(".msi"):
        return get_msi_properties(filename)[property_name]
    else:
        return get_file_properties(filename)[property_name]


def remove_outdated_binaries(version, list_extensions=["exe", "msi", "deb", "rpm", "dmg", "pkg"], filename_contains=None):
    r"""Remove files based on the version contained in his filename, failing over on file version on compatible OSes
    Args:
        version (str): version number of keeped files
        list_extensions (str or list of str): file extensions of compared files
        filename_contains (str or list of str): Part of the filename that must be contained (useful for distinguishing architecture and os)
    Returns:
        list: list of deleted files
    .. versionadded:: 2.0
    .. versionchanged:: 2.2
        Now returns removed files, now checking .exe and .msi file versions
    """
    files = []
    if type(list_extensions) != list:
        list_extensions = [list_extensions]
    if filename_contains:
        if type(filename_contains) != list:
            filename_contains = [filename_contains]
    list_extensions = ["." + ext for ext in list_extensions if ext[0] != "."]
    for file_ext in list_extensions:
        for bin_in_dir in glob.glob("*%s" % file_ext):
            if not version in bin_in_dir:
                if platform.system() == "Windows":
                    if file_ext == ".exe" or file_ext == ".msi":
                        if Version(version) == Version(get_version_from_binary(bin_in_dir, "ProductVersion")):
                            print("%s file or product version is correct (%s)" % (bin_in_dir, version))
                            continue
                remove_file(bin_in_dir)
                files.append(bin_in_dir)
            if filename_contains:
                for filename_contain in filename_contains:
                    if not filename_contain in bin_in_dir:
                        remove_file(bin_in_dir)
                        files.append(bin_in_dir)
    return [fn for fn in files]


def wait_uninstallkey_absent(key=None, max_loop=120, keywords=None, name=None):
    import time

    if len([f for f in [key, keywords, name] if f]) > 1:
        error("Arguments conflict key: %s keywords: %s name: %s" % (key, keywords, name))

    if key:
        searchparam = "key: %s" % str(key)
    elif name:
        searchparam = "name: %s" % str(name)
    else:
        searchparam = "keywords: %s" % str(keywords)
    print("Waiting for the removal of %s from Windows registry" % searchparam)
    loop = 0
    while uninstall_key_exists(key, keywords=keywords, name=name):
        loop += 1
        if loop > max_loop:
            return "timeout after %s seconds" % max_loop
        time.sleep(1)
